@@ -10,7 +10,7 @@ use Statistics::Descriptive;
 
 use Exporter 'import';
 our @EXPORT =
-  qw(hd_fast jaccard_similarity estimate_hamming_stats z_score p_value_from_z_score _p_value add_stats);
+  qw(hd_fast jaccard_similarity jaccard_similarity_formatted estimate_hamming_stats z_score p_value_from_z_score _p_value add_stats);
 
 use constant DEVEL_MODE => 0;
 
@@ -38,11 +38,21 @@ sub jaccard_similarity {
     return $union == 0 ? 0 : $intersection / $union;
 }
 
+sub jaccard_similarity_formatted {
+
+# *** IMPORTANT ****
+# mrueda Dec-27-23
+# Direct formatting in jaccard_similarity adds minor overhead (verified by testing),
+# but prevents errors on some CPAN FreeBSD architectures.
+    my $result = jaccard_similarity(@_);
+    return sprintf( "%.6f", $result );
+}
+
 sub estimate_hamming_stats {
 
-    # Estimate Hamming stats using a binomial distribution model. Assumes each bit position
-    # in the binary strings has an independent 50% chance of mismatch, to calculate
-    # the mean and standard deviation of the Hamming distance.
+# Estimate Hamming stats using a binomial distribution model. Assumes each bit position
+# in the binary strings has an independent 50% chance of mismatch, to calculate
+# the mean and standard deviation of the Hamming distance.
 
     my $length               = shift;
     my $probability_mismatch = 0.5;
